import java.util.*;

public class WordListTest {
  static int selector = 0;

  static WordList createWordList() {
    switch (selector) {
      case 0:
        return new WordList0();
      case 1:
        return new WordList1();
      default:
        return null;
    }
  }

  public static void main(String[] args) {
    char ch;

    while (true) {
      Out.println("Test functionality/efficiency/quit/WordList0/WordList1");
      Out.print("(f/e/q/0/1): ");
      ch = In.readChar();

      switch (ch) {
        case 'f':
          testFunctionality();
          break;
        case 'e':
          testEfficiency();
          break;
        case 'q':
          return;
        case '0':
          selector = 0;
          break;
        case '1':
          selector = 1;
          break;
        case '2':
          selector = 2;
          break;
        default:
          Out.println("Unknown character <" + ch + ">!");
      }
    }
  }

  static void printStrings(String[] words) {
    if (words == null) {
      return;
    }
    int size = words.length;
    if (size != 0) {
      Out.print(words[0]);
    }
    for (int i = 1; i < size; i ++) {
      Out.print(", ");
      Out.print(words[i]);
    }
    Out.println();
  }

  static boolean checkStrings(String[] w1, String[] w2) {
    if (w1 == null || w2 == null) {
      return false;
    }
    int size1 = w1.length;
    int size2 = w2.length;
    if (size1 != size2) {
      return false;
    }
    for (int i = 0; i < size1; i ++) {
      boolean found = false;
      for (int j = 0; j < size1; j ++) {
        if (w1[i].equals(w2[j])) {
          found = true;
        }
      }
      if (!found) {
        return false;
      }
    }
    return true;
  }

  static void testFunctionality() {
    boolean failed = false;
    WordList wl = createWordList();
    if (wl.nOfDifferentWords() != 0) {
      Out.println("Number of different words wrong!");
      failed = true;
    }
    double res = wl.meanWordCount();
    if (!Double.isNaN(res)) {
      Out.println("Mean word count <" + res + "> wrong!");
      failed = true;
    }
    wl.printList();
    wl.insert("Test");
    wl.printList();
    wl.insert("Test");
    wl.printList();
    wl.insert("Word");
    wl.printList();
    wl.insert("Blabla");
    wl.insert("Blabla");
    wl.insert("BLABLA");
    wl.insert("Text");
    wl.printList();
    String[] words = wl.getWords(2);
    String[] cmpWords1 = { "TEST" };
    if (!checkStrings(words, cmpWords1)) {
      Out.print("Words with frequency 2 wrong: ");
      printStrings(words);
      failed = true;
    }
    // Insert further test code here --------------
    // .
    // .
    // .
    // --------------------------------------------
    Out.println();
    Out.print("Test of functionality ");
    if (failed) {
      Out.println("failed! *******************");
    }
    else {
      Out.println("OK.");
    }
    Out.println();
  }

  // Fit here for processor speed ****************************
  static final int repeatInsert = 2;
  static final int repeatFrequency = 100000;
  static final int repeatDiffWords = 100000000;
  static final int repeatMeanWordCount = 100000000;
  static final int repeatWordsFreq = 100000;
  static final int repeatWordsStart = 10000;
  // *********************************************************

  static final int prime = 101;

  static void testEfficiency() {
    // Read words from file
    Out.print("Enter filename for efficiency test: ");
    String fn = In.readWord();
    In.open(fn);
    if (!In.done()) {
      return;
    }
    ArrayList words = new ArrayList();
    char ch = In.peek();
    while (ch != In.eof) {
      if (Character.isLetter(ch)) {
        String ident = In.readIdentifier();
        words.add(ident);
      }
      else {
        In.read();
      }
      ch = In.peek();
    }
    In.close();
    int size = words.size();
    Out.println("File " + fn + " read (" + size + " words).");

    // Test inserting words
    WordList[] wlarr = new WordList[repeatInsert];
    for (int i = 0; i < repeatInsert; i ++) {
      wlarr[i] = createWordList();
    }
    long start1 = System.currentTimeMillis();
    for (int i = 0; i < repeatInsert; i ++) {
      for (int j = 0; j < size; j ++) {
        String word = (String)words.get(j);
      }
    }
    long start2 = System.currentTimeMillis();
    for (int i = 0; i < repeatInsert; i ++) {
      for (int j = 0; j < size; j ++) {
        String word = (String)words.get(j);
        wlarr[i].insert(word);
      }
    }
    long end = System.currentTimeMillis();
    WordList wl = wlarr[0];
    Out.println("Inserting " + size + " words needs " +
            ((double)(end - start2 - (start2 - start1))) / repeatInsert +
            " milliseconds (dw = " + wl.nOfDifferentWords() + ", mean = " +
            wl.meanWordCount() + " | " + (end - start2) + "/" + (start2 - start1) + ")");

    // Test getFrequency
    start1 = System.currentTimeMillis();
    for (int i = 0; i < repeatFrequency; i ++) {
      String word = (String)words.get((i * prime) % size);
    }
    start2 = System.currentTimeMillis();
    for (int i = 0; i < repeatFrequency; i ++) {
      String word = (String)words.get((i * prime) % size);
      wl.getFrequency(word);
    }
    end = System.currentTimeMillis();
    Out.println("getFrequency needs " +
            ((double)(end - start2 - (start2 - start1))) / repeatFrequency +
            " milliseconds (" +(end - start2) + "/" + (start2 - start1) + ")");

    // Test nOfDifferentWords
    start1 = System.currentTimeMillis();
    for (int i = 0; i < repeatDiffWords; i ++) {
    }
    start2 = System.currentTimeMillis();
    for (int i = 0; i < repeatDiffWords; i ++) {
      wl.nOfDifferentWords();
    }
    end = System.currentTimeMillis();
    Out.println("nOfDifferentWords needs " +
            ((double)(end - start2 - (start2 - start1))) / repeatDiffWords +
            " milliseconds (" +(end - start2) + "/" + (start2 - start1) + ")");

    // Test meanWordCount
    start1 = System.currentTimeMillis();
    for (int i = 0; i < repeatMeanWordCount; i ++) {
    }
    start2 = System.currentTimeMillis();
    for (int i = 0; i < repeatMeanWordCount; i ++) {
      wl.meanWordCount();
    }
    end = System.currentTimeMillis();
    Out.println("meanWordCount needs " +
            ((double)(end - start2 - (start2 - start1))) / repeatMeanWordCount +
            " milliseconds (" +(end - start2) + "/" + (start2 - start1) + ")");

    // Test getWords (1)
    start1 = System.currentTimeMillis();
    for (int i = 0; i < repeatWordsFreq; i ++) {
      int freq = i % 399 + 1;
    }
    start2 = System.currentTimeMillis();
    for (int i = 0; i < repeatWordsFreq; i ++) {
      int freq = i % 399 + 1;
      wl.getWords(freq);
    }
    end = System.currentTimeMillis();
    Out.println("getWords (1-400) needs " +
            ((double)(end - start2 - (start2 - start1))) / repeatWordsFreq +
            " milliseconds (" +(end - start2) + "/" + (start2 - start1) + ")");

    // Test getWords (2)
    start1 = System.currentTimeMillis();
    for (int i = 0; i < repeatWordsFreq; i ++) {
      int freq = i % 19 + 1;
    }
    start2 = System.currentTimeMillis();
    for (int i = 0; i < repeatWordsFreq; i ++) {
      int freq = i % 19 + 1;
      wl.getWords(freq);
    }
    end = System.currentTimeMillis();
    Out.println("getWords (1-20) needs " +
            ((double)(end - start2 - (start2 - start1))) / repeatWordsFreq +
            " milliseconds (" +(end - start2) + "/" + (start2 - start1) + ")");

    // Test getWordsStartingWith
    start1 = System.currentTimeMillis();
    for (int i = 0; i < repeatWordsStart; i ++) {
      String wordStart = (String)words.get((i * prime) % size);
      wordStart = wordStart.substring(0, Math.min(i % 5 + 1, wordStart.length()));
    }
    start2 = System.currentTimeMillis();
    for (int i = 0; i < repeatWordsStart; i ++) {
      String wordStart = (String)words.get((i * prime) % size);
      wordStart = wordStart.substring(0, Math.min(i % 5 + 1, wordStart.length()));
      wl.getWordsStartingWith(wordStart);
    }
    end = System.currentTimeMillis();
    Out.println("getWordsStartingWith needs " +
            ((double)(end - start2 - (start2 - start1))) / repeatWordsStart +
            " milliseconds (" +(end - start2) + "/" + (start2 - start1) + ")");
  }
}